﻿/*
    unFormatClipboardText
    Copyright (C) 2014 - 2016 Michael Albert <info@michlstechblog.info>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace info.michlstechblog.unformatClipboardText
{
    class MACConversation
    {
        public enum MACConversationTypes { MAC_UNKNOWN_TYPE,MAC_CONV_12,MAC_CONV_16_COLON,MAC_CONV_16_DASH,MAC_CONV_16_CISCO}
        private const string REGEX_MAC_ADDRESS_12 = "^[0-9A-FDa-f]{12}$";
        private const string REGEX_MAC_CONV_16_COLON = "^([0-9A-FDa-f]{2}\\:){5}[0-9A-FDa-f]{2}$";
        private const string REGEX_MAC_CONV_16_DASH = "^([0-9A-FDa-f]{2}\\-){5}[0-9A-FDa-f]{2}$";
        private const string REGEX_MAC_CONV_16_CISCO = "^([0-9A-FDa-f]{4}\\.){2}[0-9A-FDa-f]{4}$";
        public static bool ConvertMAC12(string MAC12Address, out string MACOut, MACConversationTypes MACType) {
            MACConversationTypes InputMACType;
            MACOut = "";
            if (IsMACAddress(MAC12Address, out InputMACType)) {
                if (InputMACType == MACConversationTypes.MAC_CONV_12)
                {
                    switch (MACType)
                    {
                        case MACConversationTypes.MAC_UNKNOWN_TYPE:
                            return (false);
                        case MACConversationTypes.MAC_CONV_16_COLON:
                            //int iLoop = 0;
                            for (int iLoop = 1; iLoop <= 12; ++iLoop)
                            {
                                MACOut += MAC12Address.Substring(iLoop - 1, 1);
                                if (((iLoop % 2) == 0) && iLoop<=11)
                                {
                                    MACOut += ":";
                                }
                            }
                            break;
                        case MACConversationTypes.MAC_CONV_16_DASH:
                            //int iLoop = 0;
                            for (int iLoop = 1; iLoop <= 12; ++iLoop)
                            {
                                MACOut += MAC12Address.Substring(iLoop - 1, 1);
                                if (((iLoop % 2) == 0) && iLoop<=11)
                                {
                                    MACOut += "-";
                                }
                            }
                            break;
                        case MACConversationTypes.MAC_CONV_16_CISCO:
                            for (int iLoop = 1; iLoop <= 12; ++iLoop)
                            {
                                MACOut += MAC12Address.Substring(iLoop - 1, 1);
                                if (((iLoop % 4) == 0) && iLoop <= 11)
                                {
                                    MACOut += ".";
                                }
                            }
                            break;
                        case MACConversationTypes.MAC_CONV_12:
                            MACOut = MAC12Address;
                            break;
                        default:
                            return (false);
                    }
                    return (true);
                }
                else {
                    return (false);
                }
            }
            else {
                return (false);
            }
        }
        public static bool ConvertToMAC12(string MACAddress,out string MAC12,out MACConversationTypes MACType) {
            string sMACout = "";
            if (IsMACAddress(MACAddress,out MACType ))
            {
                sMACout=MACAddress.Replace(".", "");
                sMACout = sMACout.Replace(":", "");
                sMACout = sMACout.Replace("-", "");
                MAC12=sMACout;
                return (true);
            }
            else
            {
                MAC12 = "";
                MACType = MACConversationTypes.MAC_UNKNOWN_TYPE;
                return (false);
            }
        }
        public static bool IsMACAddress(string MACAddress, out MACConversationTypes MACType)
        { 
            if(Regex.IsMatch(MACAddress,REGEX_MAC_ADDRESS_12))
            {
                MACType=MACConversationTypes.MAC_CONV_12;
                return(true);
            }
            else if(Regex.IsMatch(MACAddress,REGEX_MAC_CONV_16_COLON))
            {
                MACType = MACConversationTypes.MAC_CONV_16_COLON;
                return (true);
            }
            else if(Regex.IsMatch(MACAddress,REGEX_MAC_CONV_16_DASH))
            {
                MACType = MACConversationTypes.MAC_CONV_16_DASH;
                return (true);
            }
            else if(Regex.IsMatch(MACAddress,REGEX_MAC_CONV_16_CISCO))
            {
                MACType = MACConversationTypes.MAC_CONV_16_CISCO;
                return (true);
            }
            else{
                MACType = MACConversationTypes.MAC_UNKNOWN_TYPE;
                return(false);
            }
        }
    }
}
