﻿/*
    unFormatClipboardText
    Copyright (C) 2014 - 2016 Michael Albert <info@michlstechblog.info>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using info.michlstechblog.unformatClipboardText;

namespace info.michlstechblog.unformatClipboardText
{
    public partial class MainForm : Form
    {
        private const string LogfileControlPreText = "Loggingfile ";
        KeyboardHook oKeyboardhook;
        Logging oLog;
        ShowLogFile oLogfileWindows;
        Control[] oLogFormControls;
        RichTextBox oLogTextBox;
        string sClipboard = "";
        int iMaxBalloonTipTextLength = 100;
        // Save Settings
        // http://stackoverflow.com/questions/453161/best-practice-to-save-application-settings-in-a-windows-forms-application
        // Clipboard OnChange
        [DllImport("user32.dll", SetLastError = true)][return: MarshalAs(UnmanagedType.Bool)]static extern bool AddClipboardFormatListener(IntPtr hwnd);
        [DllImport("user32.dll", SetLastError = true)][return: MarshalAs(UnmanagedType.Bool)]static extern bool RemoveClipboardFormatListener(IntPtr hwnd);
        private const int WM_CLIPBOARDUPDATE = 0x031D;
        protected override void WndProc(ref Message m)
        {
            try
            {
                base.WndProc(ref m);
            }
            catch(Exception Ex)
            {
                System.Diagnostics.Debug.WriteLine("Error calling WndProc(). Error:" + Ex.Message);
                return;
            }
            if (m.Msg == WM_CLIPBOARDUPDATE)
            {
                    IDataObject iData = Clipboard.GetDataObject();      // Clipboard's data.
                if (iData != null)
                {
                    RemoveClipboardFormatListener(this.Handle);
                    if (iData.GetDataPresent(DataFormats.Text))
                    {
                        // Disable Clipboard Events
                        string sClipboard = "";

                        try
                        {
                            sClipboard = (string)iData.GetData(DataFormats.Text);
                        }
                        catch (Exception Ex)
                        {
                            System.Diagnostics.Debug.WriteLine("Could not retrive data." + Ex.Message);
                            return;
                        }
                        if (enableLoggingToolStripMenuItem.Checked)
                        {
                            if (oLog != null)
                                oLog.WriteLine2LogFile("Input: " + sClipboard);
                        }
                        if (removeLeadingAndTrailingSpacesToolStripMenuItem.Checked) sClipboard = sClipboard.Trim();
                        if (convertMACToolStripMenuItem.Checked)
                        {
                            MACConversation.MACConversationTypes MACType;
                            if (MACConversation.IsMACAddress(sClipboard, out MACType))
                            {
                                MACConversation.MACConversationTypes ConvertedFrom;
                                if (MACConversation.ConvertToMAC12(sClipboard, out sClipboard, out ConvertedFrom))
                                {
                                    MACConversation.ConvertMAC12(sClipboard, out sClipboard, (MACConversation.MACConversationTypes)Properties.Settings.Default["MACCoversationType"]);
                                }
                            }
                        }
                        if (replaceModeToolStripMenuItem.Checked)
                        {
                            sClipboard = fReplaceSpecialCharcters(sClipboard);
                        }
                        if (toLowercaseToolStripMenuItem.Checked) sClipboard = sClipboard.ToLower();
                        if (toUppercaseToolStripMenuItem.Checked) sClipboard = sClipboard.ToUpper();
                        if (consolidateEmptyLinesToolStripMenuItem.Checked)
                        {
                            sClipboard = System.Text.RegularExpressions.Regex.Replace(sClipboard, @"(\r\n){2,}", "\r\n\r\n");
                            sClipboard = System.Text.RegularExpressions.Regex.Replace(sClipboard, @"(\n){2,}", "\n\n");
                        }
                        //listBox.Items.Add(sClipboard);
                        if (sClipboard.Length >= 1)
                        {
                            System.Windows.Forms.Clipboard.SetText(sClipboard, TextDataFormat.Text);
                            if (enableLoggingToolStripMenuItem.Checked)
                            {
                                if (oLog != null)
                                    oLog.WriteLine2LogFile("Output:" + sClipboard);
                            }
                            notifyIcon.BalloonTipText = sClipboard;
                            sClipboard = sClipboard.Length <= iMaxBalloonTipTextLength ? sClipboard : sClipboard.Substring(0, iMaxBalloonTipTextLength) + "...";
                            if (showBallontipToolStripMenuItem.Checked)
                                notifyIcon.ShowBalloonTip(1, "Unformat Text", sClipboard, ToolTipIcon.Info);
                            // Enalbe Clipboard Events

                        }
                    }
                    AddClipboardFormatListener(this.Handle);
                }
            }
        }
        // End Clipboard On Change
        public MainForm()
        {
            InitializeComponent();
        }
        void hook_KeyPressed(object sender, KeyPressedEventArgs e)
        {
            sClipboard = System.Windows.Forms.Clipboard.GetText().ToString();
            if (replaceModeToolStripMenuItem.Checked) {
                sClipboard=fReplaceSpecialCharcters(sClipboard);
            }
            //listBox.Items.Add(sClipboard);
            System.Windows.Forms.Clipboard.SetText(sClipboard, TextDataFormat.Text);
            notifyIcon.BalloonTipText = sClipboard;
            sClipboard = sClipboard.Length <= iMaxBalloonTipTextLength ? sClipboard : sClipboard.Substring(0, iMaxBalloonTipTextLength) + "...";
            notifyIcon.ShowBalloonTip(1, "Unformat Text", sClipboard, ToolTipIcon.Info );
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            globalHotkeyToolStripMenuItem.Checked = (bool)Properties.Settings.Default["EnableGlobalHotKey"];
            if (globalHotkeyToolStripMenuItem.Checked)
            {
                if (oKeyboardhook == null)
                    oKeyboardhook = new KeyboardHook();
                oKeyboardhook.KeyPressed += new EventHandler<KeyPressedEventArgs>(hook_KeyPressed);
                // register the control + alt + F12 combination as hot key.
                //oKeyboardhook.RegisterHotKey(info.michlstechblog.unformatClipboardText.ModifierKeys.Control | info.michlstechblog.unformatClipboardText.ModifierKeys.Control , Keys.S);
                oKeyboardhook.RegisterHotKey(info.michlstechblog.unformatClipboardText.ModifierKeys.Control, Keys.S);
                notifyIcon.Text = "Unformat Text.\nHotkey:" + oKeyboardhook.Modifier.ToString() + " + " + oKeyboardhook.Key.ToString();
            }
            //notifyIcon.Icon = info.michlstechblog.unformatClipboardText.IconExtract.ExtractIcon(Environment.SystemDirectory+"\\charmap.exe",0,false);
            notifyIcon.Icon = info.michlstechblog.unformatClipboardText.IconExtract.ExtractIcon(Environment.SystemDirectory + "\\comdlg32.dll", 0, false);
            //notifyIcon.Icon = info.michlstechblog.unformatClipboardText.IconExtract.ExtractIcon(Environment.SystemDirectory + "\\compstui.dll", 18, false);
            //notifyIcon.Text = "Unformat Text.\nHotkey:" + info.michlstechblog.unformatClipboardText.ModifierKeys.Control.ToString() + " + " + Keys.S.ToString();
            automodeToolStripMenuItem.Checked = (bool)Properties.Settings.Default["automaticMode"];
            replaceModeToolStripMenuItem.Checked = (bool)Properties.Settings.Default["replacemode"];
            toLowercaseToolStripMenuItem.Checked = (bool)Properties.Settings.Default["toLowerCase"];
            toUppercaseToolStripMenuItem.Checked = (bool)Properties.Settings.Default["toUpperCase"];
            convertMACToolStripMenuItem.Checked= (bool)Properties.Settings.Default["MACConversation"];
            showBallontipToolStripMenuItem.Checked=(bool)Properties.Settings.Default["ShowBalloonTip"];
            removeLeadingAndTrailingSpacesToolStripMenuItem.Checked=(bool)Properties.Settings.Default["RemoveSpaces"];
            consolidateEmptyLinesToolStripMenuItem.Checked = (bool)Properties.Settings.Default["ConsolidateEmptyLine"];
            UpdateMACCheckboxes(convertMACToolStripMenuItem.Checked, (MACConversation.MACConversationTypes)Properties.Settings.Default["MACCoversationType"]);
            enableLoggingToolStripMenuItem.Checked = (bool)Properties.Settings.Default["EnableLogging"];
            showLogWarningToolStripMenuItem.Checked = (bool)Properties.Settings.Default["EnableLogWarning"];
            overwriteLogfileToolStripMenuItem.Checked = (bool)Properties.Settings.Default["OverwriteLogfile"];
            deleteLogfileOnExitToolStripMenuItem.Checked = (bool)Properties.Settings.Default["DeleteLogFileOnExit"];
            scrollToEndToolStripMenuItem.Checked = (bool)Properties.Settings.Default["LogFileAutoScroll"];
            if (enableLoggingToolStripMenuItem.Checked)
            {
                LoggingControls(enableLoggingToolStripMenuItem.Checked);
                oLog = new Logging((string)Properties.Settings.Default["Logfile"], overwriteLogfileToolStripMenuItem.Checked);
                Logging.FunctionReturn Ret = oLog.OpenLogFile();
                if (!Ret.Success)
                {
                    MessageBox.Show("Error occured while opening " + oLog.LogFile + " Error:" + Ret.ErrorMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);        
                }
                else
                {
                    selectLogFileToolStripMenuItem.ToolTipText = oLog.LogFile;
                    oLog.OnLogFileChanged += LogFileChanged;
                    oLog.OnLogFileContentChanged += LogFileContentChanged;
                }
                if (showLogWarningToolStripMenuItem.Checked)
                    MessageBox.Show("Logging of clipboard is enabled. Be aware that sensitive data i.e. passwords may be stored in logfile " + oLog.LogFile + "!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning); 

            }
            if (toLowercaseToolStripMenuItem.Checked) toUppercaseToolStripMenuItem.Checked = false;
            // Clipboard On Change listener
            if (automodeToolStripMenuItem.Checked)
            {
                AddClipboardFormatListener(this.Handle);
            }
            oLogfileWindows = new ShowLogFile();
            oLogfileWindows.AutoscrollLogFile = scrollToEndToolStripMenuItem.Checked;
            oLogFormControls = oLogfileWindows.Controls.Find("richTextBoxLogFile", false);
            if (oLogFormControls.Length >= 1)
            {
                oLogTextBox = (RichTextBox)oLogFormControls[0];
                oLogTextBox.MaxLength = (int)Properties.Settings.Default["LogFileContentReadSize"];
            }
            // AppDomain.CurrentDomain.ProcessExit += OnProcessExit;
        }
        private void OnProcessExit(object sender, EventArgs e)
        {
            if (oLog != null)
            {
                oLog.CloseLogFile();
                oLog.OnLogFileChanged -= LogFileChanged;
                oLog.OnLogFileContentChanged -= LogFileContentChanged;
            }
        }
        private void UpdateMACCheckboxes(bool enableCheckboxes, MACConversation.MACConversationTypes Entry) {
            to12DigitToolStripMenuItem.Enabled = enableCheckboxes;
            toColonDigitToolStripMenuItem.Enabled = enableCheckboxes;
            toDashToolStripMenuItem.Enabled = enableCheckboxes;
            toCiscoToolStripMenuItem.Enabled = enableCheckboxes;
            to12DigitToolStripMenuItem.Checked = false;
            toColonDigitToolStripMenuItem.Checked = false;
            toDashToolStripMenuItem.Checked = false;
            toCiscoToolStripMenuItem.Checked = false;
            if (enableCheckboxes) { 
                switch (Entry)
                { 
                    case MACConversation.MACConversationTypes.MAC_CONV_12:
                        Properties.Settings.Default["MACCoversationType"] = (int)MACConversation.MACConversationTypes.MAC_CONV_12;
                        to12DigitToolStripMenuItem.Checked = true;
                        break;
                    case MACConversation.MACConversationTypes.MAC_CONV_16_COLON:
                        Properties.Settings.Default["MACCoversationType"] = (int)MACConversation.MACConversationTypes.MAC_CONV_16_COLON;
                        toColonDigitToolStripMenuItem.Checked = true;
                        break;
                    case MACConversation.MACConversationTypes.MAC_CONV_16_DASH:
                        Properties.Settings.Default["MACCoversationType"] = (int)MACConversation.MACConversationTypes.MAC_CONV_16_DASH;
                        toDashToolStripMenuItem.Checked = true;
                        break;
                    case MACConversation.MACConversationTypes.MAC_CONV_16_CISCO:
                        Properties.Settings.Default["MACCoversationType"] = (int)MACConversation.MACConversationTypes.MAC_CONV_16_CISCO;
                        toCiscoToolStripMenuItem.Checked = true;
                        break;
                    default:
                        Properties.Settings.Default["MACCoversationType"] = (int)MACConversation.MACConversationTypes.MAC_UNKNOWN_TYPE;
                        //convertMACToolStripMenuItem.Checked = false;
                        break;
                }
            }
            Properties.Settings.Default.Save();
            
        }
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (oLog != null)
            {
                oLog.CloseLogFile();
                oLog.OnLogFileChanged -= LogFileChanged;
                oLog.OnLogFileContentChanged -= LogFileContentChanged;
            }
            if (deleteLogfileOnExitToolStripMenuItem.Checked)
            {
                try
                {
                    if(oLog!=null)
                    {
                        if(System.IO.File.Exists(oLog.LogFile))
                            System.IO.File.Delete(oLog.LogFile);
                    }
                }
                catch(Exception Ex)
                {
                    MessageBox.Show("Could not delete file " + oLog.LogFile + " Error:" + Ex.Message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);        
                }
            }
            Properties.Settings.Default.Save();
            if (automodeToolStripMenuItem.Checked) 
            {
                RemoveClipboardFormatListener(this.Handle); 
            }
            if (oKeyboardhook != null)
            {
                oKeyboardhook.Dispose();
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void automodeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            automodeToolStripMenuItem.Checked = !automodeToolStripMenuItem.Checked;
            Properties.Settings.Default["automaticMode"] = automodeToolStripMenuItem.Checked;
            if (automodeToolStripMenuItem.Checked)
            {
                AddClipboardFormatListener(this.Handle);
            }
            else 
            {
                RemoveClipboardFormatListener(this.Handle);
            }
            Properties.Settings.Default.Save();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Have fun! \n\rMichael Albert\n\rhttp://michlstechblog.info\n\rVersion " + (System.Diagnostics.FileVersionInfo.GetVersionInfo((System.Reflection.Assembly.GetExecutingAssembly()).Location)).FileVersion.ToString(), "Info", MessageBoxButtons.OK, MessageBoxIcon.Information, 0);
        }
        private string fReplaceSpecialCharcters(string sIn)
        {
            sIn = sIn.Replace("„", "\"");
            sIn = sIn.Replace("“", "\"");
            sIn = sIn.Replace("–", "-");
            sIn = sIn.Replace("‘", "'");
            sIn = sIn.Replace("‚", "'");
            return sIn;
        }

        private void replaceModeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            replaceModeToolStripMenuItem.Checked = !replaceModeToolStripMenuItem.Checked;
            Properties.Settings.Default["replaceMode"] = replaceModeToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }

        private void toUppercaseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            toUppercaseToolStripMenuItem.Checked = !toUppercaseToolStripMenuItem.Checked;
            Properties.Settings.Default["toUpperCase"] = toUppercaseToolStripMenuItem.Checked;
            if (toUppercaseToolStripMenuItem.Checked) toLowercaseToolStripMenuItem.Checked = false;
            Properties.Settings.Default.Save();
        }

        private void toLowercaseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            toLowercaseToolStripMenuItem.Checked = !toLowercaseToolStripMenuItem.Checked;
            Properties.Settings.Default["toLowerCase"] = toLowercaseToolStripMenuItem.Checked;
            if (toLowercaseToolStripMenuItem.Checked) toUppercaseToolStripMenuItem.Checked = false;
            Properties.Settings.Default.Save();
        }

        private void convertMACToolStripMenuItem_Click(object sender, EventArgs e)
        {
            convertMACToolStripMenuItem.Checked = !convertMACToolStripMenuItem.Checked;
            Properties.Settings.Default["MACConversation"] = convertMACToolStripMenuItem.Checked;
            UpdateMACCheckboxes(convertMACToolStripMenuItem.Checked,(MACConversation.MACConversationTypes)Properties.Settings.Default["MACCoversationType"]);
            Properties.Settings.Default.Save();
        }

        private void to12DigitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateMACCheckboxes(convertMACToolStripMenuItem.Checked, MACConversation.MACConversationTypes.MAC_CONV_12);
        }

        private void toColonDigitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateMACCheckboxes(convertMACToolStripMenuItem.Checked, MACConversation.MACConversationTypes.MAC_CONV_16_COLON);
        }

        private void toDashToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateMACCheckboxes(convertMACToolStripMenuItem.Checked, MACConversation.MACConversationTypes.MAC_CONV_16_DASH);
        }

        private void toCiscoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateMACCheckboxes(convertMACToolStripMenuItem.Checked, MACConversation.MACConversationTypes.MAC_CONV_16_CISCO);
        }

        private void removeLeadingAndTrailingSpacesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            removeLeadingAndTrailingSpacesToolStripMenuItem.Checked = !removeLeadingAndTrailingSpacesToolStripMenuItem.Checked;
            Properties.Settings.Default["RemoveSpaces"] = removeLeadingAndTrailingSpacesToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }

        private void globalHotkeyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            globalHotkeyToolStripMenuItem.Checked = !globalHotkeyToolStripMenuItem.Checked;
            Properties.Settings.Default["EnableGlobalHotKey"] = globalHotkeyToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
            if (globalHotkeyToolStripMenuItem.Checked)
            {
                if (oKeyboardhook == null)
                    oKeyboardhook = new KeyboardHook();
                oKeyboardhook.KeyPressed += new EventHandler<KeyPressedEventArgs>(hook_KeyPressed);
                // register the control + alt + F12 combination as hot key.
                //oKeyboardhook.RegisterHotKey(info.michlstechblog.unformatClipboardText.ModifierKeys.Control | info.michlstechblog.unformatClipboardText.ModifierKeys.Control , Keys.S);
                oKeyboardhook.RegisterHotKey(info.michlstechblog.unformatClipboardText.ModifierKeys.Control, Keys.S);
                notifyIcon.Text = "Unformat Text.\nHotkey:" + oKeyboardhook.Modifier.ToString() + " + " + oKeyboardhook.Key.ToString();
            }
            else
            {
                oKeyboardhook.KeyPressed -= new EventHandler<KeyPressedEventArgs>(hook_KeyPressed);
                oKeyboardhook.UnregisterHotKeys();
            }
        }

        private void showBallontipToolStripMenuItem_Click(object sender, EventArgs e)
        {
            showBallontipToolStripMenuItem.Checked = !showBallontipToolStripMenuItem.Checked;
            Properties.Settings.Default["ShowBalloonTip"] = showBallontipToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }

        private void consolidateEmptyLinesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            consolidateEmptyLinesToolStripMenuItem.Checked = !consolidateEmptyLinesToolStripMenuItem.Checked;
            Properties.Settings.Default["ConsolidateEmptyLine"] = consolidateEmptyLinesToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }

        private void selectLogFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowFileDialog();
        }
        private void ShowFileDialog()
        {
            saveLogFileDialog.ShowDialog();
            Properties.Settings.Default["Logfile"] = saveLogFileDialog.FileName;
            Properties.Settings.Default.Save();
            if (oLog != null)
            {
                if (((string)Properties.Settings.Default["Logfile"]) != oLog.LogFile)
                {
                    string sOldLogFile = oLog.LogFile;
                    oLog.LogFile = (string)Properties.Settings.Default["Logfile"];
                    Logging.FunctionReturn Ret= oLog.OpenLogFile();
                    if (!Ret.Success)
                    {
                        MessageBox.Show("Error occured while opening " + oLog.LogFile + " Error:" + Ret.ErrorMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);        
                    }
                    else
                    {
                        selectLogFileToolStripMenuItem.ToolTipText = oLog.LogFile;
                        oLog.OnLogFileChanged += LogFileChanged;
                        oLog.OnLogFileContentChanged += LogFileContentChanged;
                    }
                    if (deleteLogfileOnExitToolStripMenuItem.Checked)
                    {
                        try
                        {
                            if (System.IO.File.Exists(sOldLogFile))
                                System.IO.File.Delete(sOldLogFile);
                        }
                        catch (Exception Ex)
                        {
                            MessageBox.Show("Could not delete Oldfile " + sOldLogFile + " Error:" + Ex.Message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        }
                    }
                }
            }
        }
        private void LogFileChanged(object Sender, Logging.LogFileChangedEventArgs EvtArgsLogfile)
        {
            selectLogFileToolStripMenuItem.ToolTipText = EvtArgsLogfile.Logfile;

        }
        private void LoggingControls(bool Enable) 
        {
            if (Enable)
            {
                showLogWarningToolStripMenuItem.Enabled = true;
                overwriteLogfileToolStripMenuItem.Enabled = true;
                deleteLogfileOnExitToolStripMenuItem.Enabled = true;
                selectLogFileToolStripMenuItem.Enabled = true;
                showLogToolStripMenuItem.Enabled = true;
                scrollToEndToolStripMenuItem.Enabled = true;
            }
            else
            {
                showLogWarningToolStripMenuItem.Enabled = false;
                overwriteLogfileToolStripMenuItem.Enabled = false;
                deleteLogfileOnExitToolStripMenuItem.Enabled = false;
                selectLogFileToolStripMenuItem.Enabled = false;
                showLogToolStripMenuItem.Enabled = false;
                scrollToEndToolStripMenuItem.Enabled = false;
            }
        }

        private void enableLoggingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            enableLoggingToolStripMenuItem.Checked = !enableLoggingToolStripMenuItem.Checked;
            Properties.Settings.Default["EnableLogging"] = enableLoggingToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
            LoggingControls(enableLoggingToolStripMenuItem.Checked);
            if(enableLoggingToolStripMenuItem.Checked)
            {
                if (String.IsNullOrEmpty((string)Properties.Settings.Default["Logfile"]))
                    ShowFileDialog();
                oLog = new Logging((string)Properties.Settings.Default["Logfile"], overwriteLogfileToolStripMenuItem.Checked);
                Logging.FunctionReturn Ret = oLog.OpenLogFile();
                if (!Ret.Success)
                {
                    MessageBox.Show("Error occured while opening " + oLog.LogFile + " Error:" + Ret.ErrorMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);        
                }
                else
                {
                    selectLogFileToolStripMenuItem.ToolTipText = oLog.LogFile;
                    oLog.OnLogFileChanged += LogFileChanged;
                    oLog.OnLogFileContentChanged += LogFileContentChanged;
                }
                if(showLogWarningToolStripMenuItem.Checked)
                    MessageBox.Show("Logging of clipboard is enabled. Be aware that sensitive data i.e. passwords may be stored in logfile " + oLog.LogFile + "!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning); 

            }
            else
            {
                if (oLog != null)
                {
                    oLog.CloseLogFile();
                    oLog.OnLogFileChanged -= LogFileChanged;
                    oLog.OnLogFileContentChanged -= LogFileContentChanged;
                }
            }
        }

        private void selectLogFileToolStripMenuItem_MouseHover(object sender, EventArgs e)
        {
            //if(oLog!=null)
            //    selectLogFileToolStripMenuItem.ToolTipText = oLog.LogFile;
        }

        private void deleteLogfileOnExitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            deleteLogfileOnExitToolStripMenuItem.Checked = !deleteLogfileOnExitToolStripMenuItem.Checked;
            Properties.Settings.Default["DeleteLogFileOnExit"] = deleteLogfileOnExitToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }

        private void showLogWarningToolStripMenuItem_Click(object sender, EventArgs e)
        {
            showLogWarningToolStripMenuItem.Checked = !showLogWarningToolStripMenuItem.Checked;
            Properties.Settings.Default["EnableLogWarning"] = showLogWarningToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }

        private void overwriteLogfileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            overwriteLogfileToolStripMenuItem.Checked = !overwriteLogfileToolStripMenuItem.Checked;
            Properties.Settings.Default["OverwriteLogfile"] = overwriteLogfileToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }
        private void LogFileContentChanged(object Sender, Logging.LogFileContentChangedEventArgs EvtArgs)
        {
            if (oLog != null)
            {
                if (oLog.oStream != null)
                {
                    if (oLogTextBox.Visible)
                    {
                        // oLogfileWindows.LastScrollpositionTextbox = oLogTextBox.SelectionStart;
                        oLogTextBox.Text = oLog.LogfileContent(oLogTextBox.MaxLength);
                        oLogfileWindows.HandleTextboxPosition();
                    }
                }
            }
        }
        private void showLogToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // oLogTextBox.Add("DS");
            if(oLog!=null)
            {
                if(oLog.oStream!=null)
                {
                    // oLogfileWindows.LastScrollpositionTextbox = oLogTextBox.SelectionStart;
                    oLogTextBox.Text = oLog.LogfileContent(oLogTextBox.MaxLength);
                }
            }
            oLogfileWindows.Show();
            oLogfileWindows.HandleTextboxPosition();
            /*if (scrollToEndToolStripMenuItem.Checked)
            {
                oLogTextBox.SelectionStart = oLogTextBox.TextLength;
                // oLogfileWindows.AutoscrollLogFile = true;
                oLogfileWindows.AutoscrollLogFile = true;
                oLogTextBox.ScrollToCaret();
            }
            else
            {
                oLogfileWindows.AutoscrollLogFile = false;
                oLogTextBox.SelectionStart = oLogfileWindows.LastScrollpositionTextbox;
                oLogTextBox.SelectionLength=1;
                oLogTextBox.ScrollToCaret();
            }*/
        }

        private void scrollToEndToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scrollToEndToolStripMenuItem.Checked = !scrollToEndToolStripMenuItem.Checked;
            if(oLogfileWindows!=null)
                oLogfileWindows.AutoscrollLogFile = scrollToEndToolStripMenuItem.Checked;
            Properties.Settings.Default["LogFileAutoScroll"] = scrollToEndToolStripMenuItem.Checked;
            Properties.Settings.Default.Save();
        }
    }
}
